local MAJOR, MINOR = "LibQuickHealth-1.0", 1
local QuickHealth, oldminor = LibStub:NewLibrary(MAJOR, MINOR)

if not QuickHealth then return end -- No Upgrade needed.
local healthTable = {};

-- Figures the health of something given its unitID
function QuickHealth:UnitHealth(unitID)
	return healthTable[UnitGUID(unitID)] or UnitHealth(unitID);
end
--local implementation starts here

-- callbackhandler
QuickHealth.events = QuickHealth.events or LibStub("CallbackHandler-1.0"):New(QuickHealth);

-- This table will contain our event handling functions.
local eventHandlers = {};

-- Register a frame to use to listen to the UNIT_HEALTH and COMBAT_LOG_EVENT_UNFLITERED events
local eventFrame = CreateFrame("Frame");
eventFrame:SetScript("OnEvent", function(self, event, ...) eventHandlers[event](...) end);
eventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
eventFrame:RegisterEvent("UNIT_HEALTH");

function eventHandlers.UNIT_HEALTH(unitID)

	local GUID = UnitGUID(unitID);
	if(UnitHealthMax(unitID) == 100) then
		healthTable[GUID] = nil;
		return;
	end
	local oldHealth = healthTable[GUID];
	local newHealth = UnitHealth(unitID);
	--ChatFrame1:AddMessage(("UNIT_HEALTH: Trigger at %.3f with %d health"):format(GetTime(), newHealth));
	if(newHealth ~= oldHealth) then
		--ChatFrame1:AddMessage(("Discrepancy between stored value and value from UnitHealth on unit %s, saved value: %d, real value: %d"):format(GUID or "", oldHealth or 0, newHealth or 0));
		healthTable[GUID] = newHealth;
		QuickHealth.events:Fire("HealthUpdated", GUID, newHealth);
	end
end

function eventHandlers.COMBAT_LOG_EVENT_UNFILTERED(timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, ...)
	if(not healthTable[destGUID]) then return; end -- Can't do anything if we don't already have a value in our cache.
	local amount;
	-- We only care about damage or healing, check for either of those and react accordingly
	if(event == "SWING_DAMAGE") then --autoattack dmg
		amount = -(...); -- putting in braces will autoselect the first arg, no need to use select(1, ...);
	elseif(event == "SPELL_PERIODIC_DAMAGE" or event == "SPELL_DAMAGE"
		or event == "DAMAGE_SPLIT" or event == "DAMAGE_SHIELD") then -- all kinds of spelldamage
		amount = -select(4, ...);
	elseif(event == "ENVIRONMENTAL_DAMAGE") then --environmental damage
		amount = -select(2, ...);
	elseif(event == "SPELL_HEAL" or event == "SPELL_PERIODIC_HEAL") then --healing
		amount = select(4, ...);
	end
	if(amount) then
		newHealth = healthTable[destGUID] + amount
		healthTable[destGUID] = newHealth;
		--ChatFrame1:AddMessage(("CombatLog: Trigger at %.3f with %d health"):format(GetTime(), newHealth));
		--ChatFrame1:AddMessage(("Health change on unit %s: %d"):format(destGUID, amount));
		QuickHealth.events:Fire("HealthUpdated", destGUID, newHealth);
	end
end